<?php

namespace Drupal\synimport\Service;

use Drupal\Core\Url;
use Drupal\pathauto\PathautoState;
use Drupal\Core\Config\FileStorage;
use Drupal\file\Entity\File;

/**
 * Defines Content Import.
 *
 * @internal
 *   For internal usage by the Commerce synimport module.
 */

/**
 * Import service.
 */
class Import {
  /**
   * Global log variable.
   *
   * @var Logger
   */
  protected $log;

  /**
   * Files service.
   *
   * @var Files
   */
  protected $filesImport;

  /**
   * Menu service.
   *
   * @var Menu
   */
  protected $menuImport;

  /**
   * Taxonomy service.
   *
   * @var Taxonomy
   */
  protected $taxonomyImport;

  /**
   * Node service.
   *
   * @var Node
   */
  protected $nodeImport;

  /**
   * Product service.
   *
   * @var Product
   */
  protected $productImport;

  /**
   * Block service.
   *
   * @var Block
   */
  protected $blockImport;

  /**
   * Service constructor.
   *
   * @param Files $filesImport
   * @param Menu $menuImport
   * @param Taxonomy $taxonomyImport
   * @param Node $nodeImport
   * @param Product $productImport
   * @param Block $blockImport
   */
  public function __construct(
    Files $filesImport,
    Menu $menuImport,
    Taxonomy $taxonomyImport,
    Node $nodeImport,
    Product $productImport,
    Block $blockImport
  ) {
    $this->filesImport = $filesImport;
    $this->menuImport = $menuImport;
    $this->taxonomyImport = $taxonomyImport;
    $this->nodeImport = $nodeImport;
    $this->productImport = $productImport;
    $this->blockImport = $blockImport;
  }

  /**
   * {@inheritdoc}
   */
  public function setLogger(Logger $log) {
    $this->log = $log;
  }

  /**
   * Import Full Content.
   */
  public function import(string $directory) {
    $this->dirIsEmpty($directory . '/contacts', "Contacts");
    $this->log->notice('Start Contacts Import');
    $this->importContacts($directory . '/contacts');
    $this->log->success("[Done Contacts Import]");

    $this->dirIsEmpty($directory . '/menu', "Menu");
    $this->log->notice('Start Menu Import');
    $this->menuImport->setLogger($this->log);
    $this->menuImport->import($directory . '/menu');
    $this->log->success("[Done Menu Import]");

    $this->dirIsEmpty($directory . '/taxonomy', "Taxonomy");
    $this->log->notice('Start Taxonomy Import');
    $this->taxonomyImport->setLogger($this->log);
    $this->taxonomyImport->import($directory . '/taxonomy');
    $this->log->success('[Done Taxonomy Import]');

    $this->dirIsEmpty($directory . '/nodes', "Node");
    $this->log->notice('Start Node Import');
    $this->nodeImport->setLogger($this->log);
    $this->nodeImport->import($directory . '/nodes');
    $this->log->success('[Done Node Import]');

    $this->dirIsEmpty($directory . '/products', "Product");
    $this->log->notice('Start Product Import');
    $this->productImport->setLogger($this->log);
    $this->productImport->import($directory . '/products');
    $this->log->success('[Done Product Import]');

    $this->dirIsEmpty($directory . '/block', "Block");
    $this->log->notice('Start Block Import');
    $this->blockImport->setLogger($this->log);
    $this->blockImport->import($directory . '/block');
    $this->log->success('[Done Block Import]');

    $this->dirIsEmpty($directory . '/synlanding', "Synlanding Configs");
    $this->log->notice('Start Setting Synlanding Configs');
    $this->setSynlandingFormBg($directory . '/synlanding');
    $this->log->success('[Done Setting Synlanding Configs]');
  }

  /**
   * Import Menu.
   */
  public function importMenu(string $directory) {
    $this->log->notice('Start Menu Import');
    $this->menuImport->setLogger($this->log);
    $this->menuImport->import($directory);
    $this->log->success("[Done Menu Import]");
  }

  /**
   * Import Taxonomy.
   */
  public function importTaxonomy(string $directory) {
    $this->log->notice('Start Taxonomy Import');
    $this->taxonomyImport->setLogger($this->log);
    $this->taxonomyImport->import($directory);
    $this->log->success('[Done Taxonomy Import]');
  }

  /**
   * Import Node.
   */
  public function importNode(string $directory) {
    $this->log->notice('Start Node Import');
    $this->nodeImport->setLogger($this->log);
    $this->nodeImport->import($directory);
    $this->log->success('[Done Node Import]');
  }

  /**
   * Import Products.
   */
  public function importProducts(string $directory) {
    $this->log->notice('Start Product Import');
    $this->productImport->setLogger($this->log);
    $this->productImport->import($directory);
    $this->log->success('[Done Product Import]');
  }

  /**
   * Import Blocks.
   */
  public function importBlocks(string $directory) {
    $this->log->notice('Start Block Import');
    $this->blockImport->setLogger($this->log);
    $this->blockImport->import($directory);
    $this->log->success('[Done Block Import]');
  }

  /**
   * Import Contacts.
   */
  public function importContacts(string $directory) {
    $sourceContacts = new FileStorage($directory);
    foreach ($sourceContacts->listAll() as $file) {
      $data = $sourceContacts->read($file);
      $path = \Drupal::service('path_alias.manager')->getPathByAlias('/contacts');
      if ($path == '/contacts') {
        $path = \Drupal::service('path_alias.manager')->getPathByAlias('/kontakty');
      }
      $params = Url::fromUri("internal:" . $path)->getRouteParameters();
      $entity_type = key($params);
      $node = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);
      $node->body->format = $data['body']['format'];
      $node->body->value = $data['body']['value'];
      $node->path = [
        'alias' => $data['path'],
        'pathauto' => PathautoState::SKIP,
      ];
      $node->save();
    }
  }

  /**
   * Set Synlanding form background.
   */
  public function setSynlandingFormBg(string $directory) {
    $sourceSynLanding = new FileStorage($directory);
    foreach ($sourceSynLanding->listAll() as $file) {
      $data = $sourceSynLanding->read($file);
      $imageId = $this->filesImport->importImages($data['form_bg_path']);
      $idFile = array_shift($imageId)['target_id'];
      $config = \Drupal::config($data['config_id']);
      \Drupal::configFactory()
        ->getEditable($data['config_id'])
        ->set('form_bg', [$idFile])
        ->save();
      $image = File::load($idFile);
      $file_usage = \Drupal::service('file.usage');
      $file_usage->add($image, 'synlanding', 'file', $idFile);
    }
  }

  /**
   * Check for empty directory.
   *
   * @throws Drupal\synimport\Classes\EmptyDirException
   *
   *   No file input.
   */
  private function dirIsEmpty(string $dirname, string $dirtype) {
    if (!file_exists($dirname)) {
      $this->log->warning("No $dirtype directory");
      return TRUE;
    }
    foreach (scandir($dirname) as $file) {
      if (!in_array($file, ['.', '..', '.svn', '.git'])) {
        return FALSE;
      }
    }
    $this->log->warning("$dirtype is empty");
    return TRUE;
  }

}
